<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is released under commercial license by Lamia Oy.
 *
 * @copyright Copyright (c) 2017 Lamia Oy (https://lamia.fi)
 * @author    Szymon Nosal <simon@lamia.fi>
 */

class ControllerExtensionPaymentVerifone extends Controller
{
    private $_errors = [];

    const VERSION = '1.3.1';

    public function __construct($registry)
    {
        parent::__construct($registry);

        $this->_load();
    }

    protected function _load()
    {
        require_once(DIR_SYSTEM . 'library/verifone/load.php');

        $this->load->model('setting/setting');
        $this->language->load('extension/payment/verifone');
        $this->load->model('customer/custom_field');
        $this->load->model('localisation/order_status');

        $storage = Verifone_Storage::getInstance();
        $storage->setSettingModel($this->model_setting_setting);
        $storage->setLanguageModel($this->language);
        $storage->setSessionModel($this->session);
        $storage->setCustomFieldsModel($this->model_customer_custom_field);
        $storage->setOrderStatusesModel($this->model_localisation_order_status);
        $storage->setVerifoneModel($this->model_extension_payment_verifone);
        $storage->setDb($this->db);

        Verifone_Upgrade::upgrade();
    }

    public function index()
    {
        $this->document->setTitle('Verifone Payment Gateway - Configuration');
        $this->document->addStyle('view/stylesheet/extension/payment/verifone.css');
        $this->document->addScript('view/javascript/extension/payment/verifone.js');


        if ($this->request->server['REQUEST_METHOD'] === 'POST') {
            $this->model_setting_setting->editSetting('payment_verifone', $this->request->post);
            $this->session->data['success'] = $this->language->get('text_success');
            $this->response->redirect($this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true));
        }

        $data['text_info'] = sprintf($this->language->get('text_info'), self::VERSION);

        $data['breadcrumbs'] = array();
        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_home'),
            'href' => $this->url->link('common/dashboard', 'user_token=' . $this->session->data['user_token'], true)
        );
        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_extension'),
            'href' => $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true)
        );
        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('heading_title'),
            'href' => $this->url->link('extension/payment/verifone', 'user_token=' . $this->session->data['user_token'], true)
        );
        $data['action'] = $this->url->link('extension/payment/verifone', 'user_token=' . $this->session->data['user_token'], true);
        $data['cancel'] = $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true);
        $data['header'] = $this->load->controller('common/header');
        $data['column_left'] = $this->load->controller('common/column_left');
        $data['footer'] = $this->load->controller('common/footer');

        $settings = new Verifone_Settings();

        $fields = [];

        foreach ($settings->getSettings() as $key => $setting) {
            $fields[$key] = $this->_prepareSettingField($key, $setting);
        }

        if (empty($fields['secret_token']['value'])) {
            $fields['secret_token']['value'] = sha1(uniqid(mt_rand(), 1));
        }

        $fields['cron_url']['value'] = HTTPS_CATALOG . 'index.php?route=extension/payment/verifone/cron&token=' . $fields['secret_token']['value'];

        // Note: since 3.0.3.5 OpenCart change method for load tempaltes. {% include %} is not available anymore.
        // Need to render field data in the controller and provide rendered html into main configuration template.
        $data['fields'] = [];

        foreach ($fields as $field) {
            $data['fields'][] = $this->load->view('extension/payment/verifone_field', ['field' => $field]);
        }

        $this->response->setOutput($this->load->view('extension/payment/verifone_3035', $data));

    }


    public function ajaxRefreshMethods()
    {

        $success = false;
        $response = null;
        $exceptionMessage = '';

        try {
            $response = Verifone_Core_PaymentMethods::refreshAvailablePaymentMethods();
            if (is_array($response)) {
                $success = true;
            }
        } catch (Exception $e) {
            $exceptionMessage = $e->getMessage();
        }

        if ($success) {
            $return = [
                'code' => 200,
                'response' => $response ? $response : '',
                'message' => $this->language->get('text_payment_method_refresh_success')
            ];
        } else {
            $return = [
                'code' => 400,
                'response' => $response ? $response : '',
                'message' => $this->language->get('text_payment_method_refresh_error') . ' ' . $exceptionMessage
            ];
        }

        $this->response->setOutput(json_encode($return));

    }

    protected function _prepareSettingField($key, $setting)
    {
        $fieldKey = 'payment_verifone_' . $key;
        $value = $this->_getValue($fieldKey);

        if (empty($value) && isset($setting['default']) && !empty($setting['default'])) {
            $value = $setting['default'];
        }

        $tooltip = null;

        if (isset($setting['tooltip']) && $setting['tooltip']) {
            $tooltip = $this->language->get('entry_tooltip_' . $key);
        }

        $field = [
            'key' => $fieldKey,
            'value' => $value,
            'label' => $this->language->get('entry_title_' . $key),
            'tooltip' => $tooltip,
            'type' => $setting['type'],
            'readonly' => isset($setting['readonly']) && $setting['readonly'],
            'class' => $setting['class'] ?? ''
        ];

        if ($setting['type'] === 'select' || $setting['type'] === 'multiselect') {
            foreach ($setting['options'] as $option) {
                $field['options'][] = [
                    'value' => $option['value'],
                    'label' => $this->language->get($option['label'])
                ];
            }
        }

        if ($setting['type'] === 'url') {
            $field['href'] = $this->url->link($setting['route'], 'user_token=' . $this->session->data['user_token'], true);
        }

        if ($setting['type'] === 'generate_keys') {
            $generateLiveLink = $this->language->get('entry_generate_live_keys');
            $generateTestLink = $this->language->get('entry_generate_test_keys');

            $generateLiveDesc = $this->language->get('entry_generate_live_info');
            $generateTestDesc = $this->language->get('entry_generate_test_info');

            $confirmation_msg1 = $this->language->get('entry_generate_confirm1');
            $confirmation_msg2 = $this->language->get('entry_generate_confirm2');

            $format = '%s <br/>%s<br/><br/>%s<br/>%s <span class="confirm hidden">%s</span>';

            $field['generateLiveLink'] = $generateLiveLink;
            $field['generateTestLink'] = $generateTestLink;
            $field['generateLiveDesc'] = $generateLiveDesc;
            $field['generateTestDesc'] = $generateTestDesc;

            $field['confirmation'] = $confirmation_msg1 . "\n\n" . $confirmation_msg2;

            $field['href'] = $this->url->link($setting['route'], 'user_token=' . $this->session->data['user_token'], true);

        }

        if ($setting['type'] === 'verifone-display-summary') {
            $field['configurationData'] = Verifone_Summary::getConfigurationDataForDisplay();
            $field['buttonLabel'] = $this->language->get('entry_title_display_summary');
            $field['modalHeader'] = $this->language->get('entry_title_summary');

            $this->load->model('localisation/currency');
            $currency = $this->model_localisation_currency->getCurrencyByCode(Verifone_System::getCurrencyCode());

            if($currency['value'] != 1) {
                $message = $this->language->get('currency_rate_message');
                $currentRateMessage = $this->language->get('currency_rate_current');
                $correctRateMessage = $this->language->get('currency_rate_correct');
                $field['note'] = sprintf($message . '<br/>' . $currentRateMessage . '<br/>' . $correctRateMessage, $currency['value'], '1.00');
            }
        }

        return $field;
    }

    protected function _addError($key, $message)
    {
        $this->_errors[$key] = $message;
    }

    protected function _getError($key)
    {
        if (isset($this->_errors[$key])) {
            return $this->_errors[$key];
        }

        return '';
    }

    protected function _getValue($key)
    {
        if (isset($this->request->post[$key])) {
            return $this->request->post[$key];
        } else {
            return $this->config->get($key);
        }
    }

    public function install()
    {
        $this->load->model('extension/payment/verifone');

        $this->model_extension_payment_verifone->createDatabaseTables();
    }

    public function uninstall()
    {

    }

    public function order()
    {

        if ($this->config->get('payment_verifone_status')) {

            $this->load->model('extension/payment/verifone');
            $this->load->model('sale/order');

            $verifoneOrder = $this->model_extension_payment_verifone->getOrder($this->request->get['order_id']);
            $order = $this->model_sale_order->getOrder($this->request->get['order_id']);

            if (!empty($verifoneOrder) && $order['order_status_id'] === Verifone_Config::getInstance()->getOrderStatusSuccess()) {
                $this->load->language('extension/payment/verifone');

                $verifoneOrder['total_released'] = $this->model_extension_payment_verifone->getTotalReleased($verifoneOrder['verifone_order_id']);

                $verifoneOrder['total_formatted'] = $this->currency->format($verifoneOrder['total'], $verifoneOrder['currency_code'], false);
                $verifoneOrder['total_released_formatted'] = $this->currency->format($verifoneOrder['total_released'], $verifoneOrder['currency_code'], false);

                $data['verifone_order'] = $verifoneOrder;

                $data['order_id'] = $this->request->get['order_id'];

                $data['user_token'] = $this->request->get['user_token'];

                return $this->load->view('extension/payment/verifone_order', $data);
            } else {
                $data['order_id'] = $this->request->get['order_id'];
                $data['user_token'] = $this->request->get['user_token'];

                return $this->load->view('extension/payment/verifone_paymentstatus', $data);
            }
        }
    }

    public function refund()
    {
        $this->load->language('extension/payment/verifone');
        $json = array();

        if (isset($this->request->post['order_id']) && !empty($this->request->post['order_id'])) {
            $this->load->model('extension/payment/verifone');
            $this->load->model('extension/payment/verifoneOrder');

            $verifoneOrder = $this->model_extension_payment_verifone->getOrder($this->request->post['order_id']);

            $totalReleased = $this->model_extension_payment_verifone->getTotalReleased($verifoneOrder['verifone_order_id']);

            if ($totalReleased < $this->request->post['amount']) {
                $json['error'] = true;
                $json['msg'] = $this->language->get('text_refund_unable');

                return $this->response->setOutput(json_encode($json));
            }

            $refundResponse = $this->model_extension_payment_verifone->refund($this->request->post['order_id'], $this->request->post['amount']);

            if ($refundResponse['error'] == false) {
                $this->model_extension_payment_verifone->addTransaction($verifoneOrder['verifone_order_id'], 'refund', $this->request->post['amount'] * -1);
                $this->model_extension_payment_verifoneOrder->addOrderHistory($this->request->post['order_id'], null, $refundResponse['message']);

                $totalRefunded = $this->model_extension_payment_verifone->getTotalRefunded($verifoneOrder['verifone_order_id']);
                $totalReleased = $this->model_extension_payment_verifone->getTotalReleased($verifoneOrder['verifone_order_id']);

                if ($totalReleased == 0) {
                    $this->model_extension_payment_verifone->updateRefundStatus($verifoneOrder['verifone_order_id'], 1);
                }

                $json['msg'] = $refundResponse['message'];
                $json['data'] = array();
                $json['data']['created'] = date("Y-m-d H:i:s");
                $json['data']['amount'] = $this->currency->format(($this->request->post['amount'] * -1), $verifoneOrder['currency_code'], false);
                $json['data']['total_released'] = $this->currency->format($totalReleased, $verifoneOrder['currency_code'], false);
                $json['data']['total_refund'] = $this->currency->format($totalRefunded, $verifoneOrder['currency_code'], false);
                $json['data']['refund_status'] = 1;
                $json['data']['canRefund'] = $totalReleased > 0;
                $json['error'] = false;
            } else {
                $json['error'] = true;
                $json['msg'] = (isset($refundResponse['message']) && !empty($refundResponse['message'])) ? (string)$refundResponse['message'] : $this->language->get('text_refund_error_unable');
            }
        } else {
            $json['error'] = true;
            $json['msg'] = $this->language->get('text_error_missingdata');
        }

        if (empty($json['msg'])) {
            $json['msg'] = $this->language->get('text_refund_message_error');
        }

        return $this->response->setOutput(json_encode($json));
    }

    public function paymentStatus()
    {
        $this->load->language('extension/payment/verifone');

        $json = array();

        if (isset($this->request->post['order_id']) && !empty($this->request->post['order_id'])) {
            $this->load->model('extension/payment/verifone');
            $this->load->model('extension/payment/verifoneOrder');
            $this->load->model('sale/order');

            $this->_load();

            Verifone_Storage::getInstance()->setOrderModel($this->model_extension_payment_verifoneOrder);
            Verifone_Storage::getInstance()->setVerifoneOrderStatus($this->model_extension_payment_verifoneOrder);

            $order = $this->model_sale_order->getOrder($this->request->post['order_id']);

            $result = Verifone_Observer::executeCheckOrderPaymentAction($order);

            $this->model_extension_payment_verifoneOrder->addOrderHistory($this->request->post['order_id'], $result['status'], $result['message']);

            $json['error'] = false;
            $json['msg'] = $this->language->get('text_error_refreshpage');

        } else {
            $json['error'] = true;
            $json['msg'] = $this->language->get('text_error_missingdata');
        }

        $this->response->setOutput(json_encode($json));
    }

    public function ajaxGenerateKeys()
    {

        $result = Verifone_Core_Generator::generateKeys();

        $messages = [];

        foreach ($result['messages'] as $message) {
            $messages[] = $this->language->get($message);
        }

        if ($result['success']) {
            $return = [
                'code' => 200,
                'messages' => $messages
            ];
        } else {
            $return = [
                'code' => 400,
                'messages' => $messages
            ];
        }

        $this->response->setOutput(json_encode($return));

    }

}
