<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is released under commercial license by Lamia Oy.
 *
 * @copyright Copyright (c) 2017 Lamia Oy (https://lamia.fi)
 * @author    Szymon Nosal <simon@lamia.fi>
 */

class ModelExtensionPaymentVerifone extends Model
{

    public function __construct($registry)
    {
        parent::__construct($registry);

        require_once(DIR_SYSTEM . 'library/verifone/load.php');
        $this->load->model('setting/setting');
        $this->language->load('extension/payment/verifone');
        $this->load->model('customer/custom_field');
        $this->load->model('localisation/order_status');


        $storage = Verifone_Storage::getInstance();
        $storage->setSettingModel($this->model_setting_setting);
        $storage->setLanguageModel($this->language);
        $storage->setCustomFieldsModel($this->model_customer_custom_field);
        $storage->setOrderStatusesModel($this->model_localisation_order_status);
    }

    public function createDatabaseTables()
    {
        $this->db->query("
			CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "verifone_order` (
			  `verifone_order_id` INT(11) NOT NULL AUTO_INCREMENT,
			  `order_id` INT(11) NOT NULL,
			  `order_ext_id` VARCHAR(50),
			  `date_added` DATETIME NOT NULL,
			  `date_modified` DATETIME NOT NULL,
			  `refund_status` INT(1) DEFAULT NULL,
			  `currency_code` CHAR(3) NOT NULL,
			  `total` DECIMAL( 10, 2 ) NOT NULL,
              `payment_method` VARCHAR(32) DEFAULT NULL,
              `masked_pan_number` VARCHAR(32) DEFAULT NULL,
			  PRIMARY KEY (`verifone_order_id`)
			) ENGINE=MyISAM DEFAULT COLLATE=utf8_general_ci;");

        $this->db->query("
			CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "verifone_order_transaction` (
			  `verifone_order_transaction_id` INT(11) NOT NULL AUTO_INCREMENT,
			  `verifone_order_id` INT(11) NOT NULL,
			  `date_added` DATETIME NOT NULL,
			  `type` ENUM('payment', 'refund') DEFAULT NULL,
			  `amount` DECIMAL( 10, 2 ) NOT NULL,
			  PRIMARY KEY (`verifone_order_transaction_id`)
			) ENGINE=MyISAM DEFAULT COLLATE=utf8_general_ci;");

        $this->db->query("
			CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "verifone_card` (
			  `card_id` INT(11) NOT NULL AUTO_INCREMENT,
			  `customer_id` INT(11) NOT NULL,
			  `token` VARCHAR(50) NOT NULL UNIQUE,
			  `digits` VARCHAR(22) NOT NULL,
			  `expiry` VARCHAR(5) NOT NULL,
			  `type` VARCHAR(50) NOT NULL,
			  PRIMARY KEY (`card_id`)
			) ENGINE=MyISAM DEFAULT COLLATE=utf8_general_ci;");
    }

    public function dropDatabaseTables()
    {

    }

    public function addOrder($orderInfo, $orderExtId, $paymentMethod)
    {
        $this->db->query("
            INSERT INTO `" . DB_PREFIX . "verifone_order` SET 
            `order_id` = '" . (int)$orderInfo['order_id'] . "', 
            `order_ext_id` = '" . $this->db->escape($orderExtId) . "', 
            `date_added` = '" . $orderInfo['date_added'] . "', 
            `date_modified` = '" . $orderInfo['date_modified'] . "', 
            `currency_code` = '" . $this->db->escape($orderInfo['currency_code']) . "', 
            `total` = '" . $this->currency->format($orderInfo['total'], $orderInfo['currency_code'], false, false) . "',
            `payment_method` = '" . $this->db->escape($paymentMethod) . "'
            ");

        return $this->db->getLastId();
    }

    public function getOrder($orderId)
    {

        $qry = $this->db->query("SELECT * FROM `" . DB_PREFIX . "verifone_order` WHERE `order_id` = '" . (int)$orderId . "' LIMIT 1");

        if ($qry->num_rows) {
            $order = $qry->row;
            $order['transactions'] = $this->getTransactions($order['verifone_order_id'], $qry->row['currency_code']);

            return $order;
        } else {
            return false;
        }
    }

    private function getTransactions($verifoneOrderId, $currencyCode)
    {
        $query = $this->db->query("SELECT * FROM `" . DB_PREFIX . "verifone_order_transaction` WHERE `verifone_order_id` = '" . (int)$verifoneOrderId . "'");

        $transactions = array();
        if ($query->num_rows) {
            foreach ($query->rows as $row) {
                $row['amount'] = $this->currency->format($row['amount'], $currencyCode, false);
                $transactions[] = $row;
            }
            return $transactions;
        } else {
            return false;
        }
    }

    public function addTransaction($verifoneOrderId, $type, $total)
    {
        $this->db->query("INSERT INTO `" . DB_PREFIX . "verifone_order_transaction` SET `verifone_order_id` = '" . (int)$verifoneOrderId . "', `date_added` = now(), `type` = '" . $this->db->escape($type) . "', `amount` = '" . (double)$total . "'");
    }

    public function getTotalReleased($verifoneOrderId)
    {
        $query = $this->db->query("SELECT SUM(`amount`) AS `total` FROM `" . DB_PREFIX . "verifone_order_transaction` WHERE `verifone_order_id` = '" . (int)$verifoneOrderId . "' AND (`type` = 'payment' OR `type` = 'refund')");

        return (double)$query->row['total'];
    }

    public function getTotalRefunded($verifoneOrderId)
    {
        $query = $this->db->query("SELECT SUM(`amount`) AS `total` FROM `" . DB_PREFIX . "verifone_order_transaction` WHERE `verifone_order_id` = '" . (int)$verifoneOrderId . "' AND 'refund'");

        return (double)$query->row['total'];
    }

    public function updateRefundStatus($verifoneOrderId, $status) {
        $this->db->query("UPDATE `" . DB_PREFIX . "verifone_order` SET `refund_status` = '" . (int)$status . "' WHERE `verifone_order_id` = '" . (int)$verifoneOrderId . "'");
    }

    public function refund($orderId, $amount)
    {
        $verifoneOrder = $this->getOrder($orderId);

        $return = [
            'error' => false,
            'message' => ''
        ];

        if (!empty($verifoneOrder) && $verifoneOrder['refund_status'] != 1 && !empty($verifoneOrder['order_ext_id'])) {

            try {

                $result = Verifone_Core_Refund::refund($verifoneOrder, (float)$amount);

                if ($result) {

                    /** @var Verifone\Core\DependencyInjection\Service\TransactionImpl $body */
                    $body = $result->getFull();

                    $refundMessage = sprintf($this->language->get('text_refund_message'), (float)$amount, $body->getNumber());
                    $return['message'] = $refundMessage;
                    return $return;
                } else {
                    $return['message'] = $this->language->get('text_refund_message_error');
                }


            } catch (Exception $e) {
                $error = $e->getMessage();
                $return['error'] = true;
                $return['message'] = $error;
            }

        } else {
            $return['error'] = true;
            $return['message'] = $this->language->get('text_refund_message_error');
        }

        return $return;
    }

}