<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is released under commercial license by Lamia Oy.
 *
 * @copyright Copyright (c) 2017 Lamia Oy (https://lamia.fi)
 * @author    Szymon Nosal <simon@lamia.fi>
 */

class ControllerExtensionPaymentVerifone extends Controller
{

    public function __construct($registry)
    {
        parent::__construct($registry);

        require_once(DIR_SYSTEM . 'library/verifone/load.php');

        $this->load->model('setting/setting');
        $this->language->load('extension/payment/verifone');
        $this->load->model('checkout/order');
        $this->load->model('account/custom_field');
        $this->load->model('localisation/order_status');
        $this->load->model('extension/payment/verifone');
        $this->load->model('extension/payment/verifoneOrderStatus');

        $storage = Verifone_Storage::getInstance();
        $storage->setSettingModel($this->model_setting_setting);
        $storage->setLanguageModel($this->language);
        $storage->setSessionModel($this->session);
        $storage->setUrlModel($this->url);
        $storage->setCustomFieldsModel($this->model_account_custom_field);
        $storage->setOrderModel($this->model_checkout_order);
        $storage->setOrderStatusesModel($this->model_localisation_order_status);
        $storage->setVerifoneModel($this->model_extension_payment_verifone);
        $storage->setCustomerModel($this->customer);
        $storage->setDb($this->db);
        $storage->setVerifoneOrderStatus($this->model_extension_payment_verifoneOrderStatus);
    }

    public function index()
    {
        $data['button_confirm'] = $this->language->get('text_pay_with_verifone');
        $data['action'] = $this->url->link('extension/payment/verifone/pay', '', true);

        $allowSaveCC = Verifone_Config::getInstance()->isAllowToSaveCC();
        $data['allowSaveCC'] = $allowSaveCC;

        $data['rememberPaymentMethodInfo'] = Verifone_Config::getInstance()->getRememberCCInfo();

        $paymentMethods = Verifone_PaymentMethods::getAvailablePaymentMethods();
        $data['paymentMethods'] = $paymentMethods;

        $justAll = count($paymentMethods) === 1 && isset($paymentMethods[Verifone_PaymentMethods::TYPE_ALL]);

        $customerId = $this->customer->isLogged();

        $cardAvailable = false;

        foreach ($paymentMethods as $method) {
            if ($method['type'] === Verifone_PaymentMethods::TYPE_CARD) {
                $cardAvailable = true;
                break;
            }
        }

        if ($allowSaveCC && $customerId && $cardAvailable) {
            $savedPaymentMethods = [];

            foreach ($this->model_extension_payment_verifone->getCards($customerId) as $card) {

                $number = explode('*', $card['digits']);
                $name = '*********' . end($number) . ' (' . $card['expiry'] . ')';

                $savedPaymentMethods[] = [
                    'code' => $card['token'],
                    'displayName' => $name
                ];
            }

            $data['savedPaymentMethods'] = $savedPaymentMethods;

        }

        $data['justAll'] = $justAll;
        $data['justAllMessage'] = $this->language->get('verifone_justAllMessage');
        $data['actionDeleteCard'] = $this->url->link('extension/payment/verifone/deleteCard', '', true);

        return $this->load->view('extension/payment/verifone', $data);
    }

    public function pay()
    {
        if ($this->session->data['payment_method']['code'] !== 'verifone') {
            return;
        }

        $this->language->load('extension/payment/verifone');
        $this->load->model('extension/payment/verifone');
        $this->load->model('account/order');

        $orderData = $this->model_checkout_order->getOrder($this->session->data['order_id']);

        $orderData['totals'] = [];

        foreach ($this->model_checkout_order->getOrderTotals($this->session->data['order_id']) as $total) {
            $orderData['totals'][$total['code']] = $total['value'];
        }

        $paymentMethod = null;
        $savePaymentMethod = null;

        $post = $this->request->post;
        if (isset($post['verifone-save-payment-method']) && $post['verifone-save-payment-method']) {
            $savePaymentMethod = true;
        }

        if (isset($post['verifone-payment-method']) && $post['verifone-payment-method']) {
            $paymentMethod = $post['verifone-payment-method'];
        }

        $items = $this->model_account_order->getOrderProducts($orderData['order_id']);

        $data = Verifone_Data::collectData($orderData, $items, $paymentMethod, $savePaymentMethod);

        // one click payment
        // Due to PSD/2 regulation payment must be redirect to the payment provider page.
        // Is not possible to make payment request by S2S integration

        $formData = Verifone_Core_Payment::preparePaymentRequestFormData($data);

        $action = $formData['action'];
        unset($formData['action']);

        $context = [
            'action' => $action,
            'redirectMessage' => $this->language->get('text_redirect'),
            'formData' => $formData
        ];

//        $this->model_checkout_order->addOrderHistory($this->session->data['order_id'], Verifone_Config::getInstance()->getOrderStatusPendingPayment());
        $this->model_extension_payment_verifoneOrderStatus->addOrderHistory($this->session->data['order_id'], Verifone_Config::getInstance()->getOrderStatusPendingPayment());
        $this->_clearSession();

        $this->response->setOutput($this->load->view('extension/payment/verifone_redirect', $context));

    }

    protected function _clearSession()
    {
        if (isset($this->session->data['order_id'])) {
            $this->cart->clear();

            unset($this->session->data['shipping_method']);
            unset($this->session->data['shipping_methods']);
            unset($this->session->data['payment_method']);
            unset($this->session->data['payment_methods']);
            unset($this->session->data['guest']);
            unset($this->session->data['comment']);
            unset($this->session->data['order_id']);
            unset($this->session->data['coupon']);
            unset($this->session->data['reward']);
            unset($this->session->data['voucher']);
            unset($this->session->data['vouchers']);
            unset($this->session->data['totals']);
        }
    }

    public function success()
    {
        $response = Verifone_Payment::processSuccess('success');

        $this->response->redirect($this->url->link($response['return_url']));
        return;
    }

    public function successDelayed()
    {
        $response = Verifone_Payment::processSuccess('success');

        if($response['error']) {
            header("HTTP/1.1 400 OK");
        } else {
            // no session, i.e. late POST from the payment system. We must signal 200 OK.
            header("HTTP/1.1 200 OK");
        }

        die('<html><head><meta http-equiv="refresh" content="0;url=' . Verifone_Urls::getCheckoutUrl() . '"></head></html>');
    }

    public function cancel()
    {
        $this->cancelPayment('cancel');
    }

    public function error()
    {
        $this->cancelPayment('error');
    }

    public function rejected()
    {
        $this->cancelPayment('rejected');
    }

    public function expired()
    {
        $this->cancelPayment('expired');
    }

    public function cancelPayment($status)
    {
        $response = Verifone_Payment::processFail($status);

        $reorderError = $this->model_extension_payment_verifone->reorder($response['order_id']);

        if ($reorderError) {
            $this->session->data['error'] = $this->language->get('text_cant_restore_cart');
        }

        $this->response->redirect($this->url->link($response['return_url']));
        return;
    }

    public function deleteCard()
    {
        $customerId = $this->customer->isLogged();
        $token = $this->request->post['token'];

        $ret = [
            'error' => true,
            'message' => $this->language->get('text_remove_card_error')
        ];

        if (!$customerId || !$token) {
            return $this->response->setOutput(json_encode($ret));
        }

        $card = $this->model_extension_payment_verifone->getCard($token, $customerId);

        if (empty($card)) {
            return $this->response->setOutput(json_encode($ret));
        }

        if ($this->model_extension_payment_verifone->deleteCard($token)) {

            if (Verifone_Core_SavedPaymentMethods::deleteCard($card['token'])) {
                $ret['error'] = false;
                $ret['message'] = $this->language->get('text_remove_card_success');

                return $this->response->setOutput(json_encode($ret));
            }
        }

        return $this->response->setOutput(json_encode($ret));

    }

    public function cron()
    {
        if ($this->request->get['token'] === $this->config->get('payment_verifone_secret_token')) {

            $this->load->model('extension/payment/verifone');

            $orders = $this->model_extension_payment_verifone->getOrdersForCheckPayment();

            foreach ($orders as $order) {
                if (Verifone_Payment::transactionsCanBeCheck($order)) {
                    $result = Verifone_Observer::executeCheckOrderPaymentAction($order, true);
//                    $this->model_extension_payment_verifone->addOrderHistory($order['order_id'], $result['status'], $result['message']);
                    $this->model_extension_payment_verifoneOrderStatus->addOrderHistory($order['order_id'], $result['status'], $result['message']);
                }
            }
        }
    }
}