<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is released under commercial license by Lamia Oy.
 *
 * @copyright Copyright (c) 2017 Lamia Oy (https://lamia.fi)
 * @author    Szymon Nosal <simon@lamia.fi>
 */

class ModelExtensionPaymentVerifone extends Model
{

    public function __construct($registry)
    {
        parent::__construct($registry);

        require_once(DIR_SYSTEM . 'library/verifone/load.php');
        $this->load->model('setting/setting');
        $this->language->load('extension/payment/verifone');
        $this->load->model('account/custom_field');
        $this->load->model('localisation/order_status');


        $storage = Verifone_Storage::getInstance();
        $storage->setSettingModel($this->model_setting_setting);
        $storage->setLanguageModel($this->language);
        $storage->setCustomFieldsModel($this->model_account_custom_field);
        $storage->setOrderStatusesModel($this->model_localisation_order_status);
        $storage->setCustomerModel($this->customer);
    }

    public function getMethod($address, $total)
    {
        $this->load->language('extension/payment/verifone');

        $config = Verifone_Config::getInstance();

        $status = $config->isEnabled() && $total > 0.00;

        if ($config->getMinOrderTotal() && !empty($config->getMinOrderTotal()) && $total < $config->getMinOrderTotal()) {
            $status = false;
        }

        if ($config->getMaxOrderTotal() && !empty($config->getMaxOrderTotal()) && $total > $config->getMaxOrderTotal()) {
            $status = false;
        }

        $method_data = array();
        if ($status) {
            $method_data = array(
                'code' => 'verifone',
                'title' => $this->config->get('payment_verifone_title'),
                'terms' => '',
                'sort_order' => $this->config->get('payment_verifone_sort_order')
            );
        }
        return $method_data;
    }

    public function addNewTransaction($orderId, $transactionId, $paymentMethod)
    {
        $query = 'INSERT INTO ' . DB_PREFIX . 'verifone_payment VALUES (NULL, "' . (int)$orderId . '", "' . $this->db->escape($transactionId) . '", "' . $this->db->escape($paymentMethod) . '")';
        return $this->db->query($query);
    }

    public function getTransactionsByOrderId($orderId)
    {
        $query = 'SELECT * FROM ' . DB_PREFIX . 'verifone_payment WHERE order_id ="' . (int)$orderId . '"';
        $result = $this->db->query($query)->row;
        if (!$result) {
            return null;
        }
        return $result;
    }

    public function addOrder($orderInfo, $orderExtId, $paymentMethod)
    {
        $this->db->query("
            INSERT INTO `" . DB_PREFIX . "verifone_order` SET 
            `order_id` = '" . (int)$orderInfo['order_id'] . "', 
            `order_ext_id` = '" . $this->db->escape($orderExtId) . "', 
            `date_added` = '" . $orderInfo['date_added'] . "', 
            `date_modified` = '" . $orderInfo['date_modified'] . "', 
            `currency_code` = '" . $this->db->escape($orderInfo['currency_code']) . "', 
            `total` = '" . $orderInfo['total'] . "',
            `payment_method` = '" . $this->db->escape($paymentMethod) . "'
            ");

        return $this->db->getLastId();
    }

    public function getOrder($orderId)
    {
        $query = $this->db->query("SELECT * FROM `" . DB_PREFIX . "verifone_order` WHERE `order_id` = '" . (int)$orderId . "' LIMIT 1");

        if ($query->num_rows) {
            $order = $query->row;
            $order['transactions'] = $this->getTransactions($order['verifone_order_id']);

            return $order;
        } else {
            return false;
        }
    }

    public function addTransaction($verifoneOrderId, $type, $transactionAmount)
    {
        $this->db->query("INSERT INTO `" . DB_PREFIX . "verifone_order_transaction` SET `verifone_order_id` = '" . (int)$verifoneOrderId . "', `date_added` = now(), `type` = '" . $this->db->escape($type) . "', `amount` = '" . $transactionAmount . "'");
    }

    public function getTransactions($verifoneOrderId)
    {
        $query = $this->db->query("SELECT * FROM `" . DB_PREFIX . "verifone_order_transaction` WHERE `verifone_order_id` = '" . (int)$verifoneOrderId . "'");

        if ($query->num_rows) {
            return $query->rows;
        } else {
            return false;
        }
    }

    public function addCard($card_data)
    {
        try {
            $this->db->query(
                "INSERT into " . DB_PREFIX . "verifone_card SET 
            customer_id = '" . $this->db->escape($card_data['customer_id']) . "', 
            digits = '" . $this->db->escape($card_data['digits']) . "', 
            expiry = '" . $this->db->escape($card_data['expiry']) . "', 
            type = '" . $this->db->escape($card_data['type']) . "', 
            token = '" . $this->db->escape($card_data['token']) . "'
            ");
        } catch (Exception $e) {
            // not allowed for add duplicate - unique index on token
        }
    }

    public function deleteCard($token)
    {
        $this->db->query("DELETE FROM " . DB_PREFIX . "verifone_card WHERE customer_id = '" . $this->customer->isLogged() . "' AND token = '" . $this->db->escape($token) . "'");

        if ($this->db->countAffected() > 0) {
            return true;
        } else {
            return false;
        }
    }

    public function getCard($token, $customerId)
    {
        $query = $this->db->query("SELECT * FROM " . DB_PREFIX . "verifone_card WHERE token LIKE " . $token . " AND customer_id = " . (int)$customerId);

        return $query->row;
    }

    public function getCards($customerId)
    {
        $query = $this->db->query("SELECT * FROM " . DB_PREFIX . "verifone_card WHERE customer_id = " . (int)$customerId);

        return $query->rows;
    }

    public function getVerifoneOrder($verifoneOrderId)
    {
        $query = $this->db->query("SELECT * FROM " . DB_PREFIX . "verifone_order WHERE order_code = " . (int)$verifoneOrderId);
        return $query->row;
    }

    public function reorder($order_id = 0)
    {
        $this->load->language('account/order');
        $this->load->model('account/order');

        $order_info = $this->model_account_order->getOrder($order_id);
        $error = false;

        if ($order_info) {

            $order_products = $this->model_account_order->getOrderProducts($order_id);

            if ($order_products) {
                $this->load->model('catalog/product');

                foreach ($order_products as $product_info) {

                    if ($product_info) {
                        $option_data = array();

                        $order_options = $this->model_account_order->getOrderOptions($order_id, $product_info['order_product_id']);

                        foreach ($order_options as $order_option) {
                            if ($order_option['type'] == 'select' || $order_option['type'] == 'radio' || $order_option['type'] == 'image') {
                                $option_data[$order_option['product_option_id']] = $order_option['product_option_value_id'];
                            } elseif ($order_option['type'] == 'checkbox') {
                                $option_data[$order_option['product_option_id']][] = $order_option['product_option_value_id'];
                            } elseif ($order_option['type'] == 'text' || $order_option['type'] == 'textarea' || $order_option['type'] == 'date' || $order_option['type'] == 'datetime' || $order_option['type'] == 'time') {
                                $option_data[$order_option['product_option_id']] = $order_option['value'];
                            } elseif ($order_option['type'] == 'file') {
                                $option_data[$order_option['product_option_id']] = $this->encryption->encrypt($this->config->get('config_encryption'), $order_option['value']);
                            }
                        }

                        $this->cart->add($product_info['product_id'], $product_info['quantity'], $option_data);

                        unset($this->session->data['shipping_method']);
                        unset($this->session->data['shipping_methods']);
                        unset($this->session->data['payment_method']);
                        unset($this->session->data['payment_methods']);
                    } else {
                        $error = true;
                    }
                }
            }
        }

        return $error;

    }

    public function getLastOrderHistory($orderId)
    {
        $query = $this->db->query("
            SELECT * FROM `" . DB_PREFIX . "order_history` 
            WHERE 
                `order_id` = " . $orderId .
            " ORDER BY `order_history_id` DESC");

        return $query->row;
    }

    public function getOrdersForCheckPayment()
    {
        $statusId = Verifone_Config::getInstance()->getOrderStatusPendingPayment();
        $query = $this->db->query("
            SELECT * FROM `" . DB_PREFIX . "order` 
            WHERE 
                `date_added` > DATE_SUB(now(), INTERVAL 2 HOUR) AND
                `order_status_id` = " . $statusId);

        return $query->rows;
    }

}