<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is released under commercial license by Lamia Oy.
 *
 * @copyright Copyright (c) 2017 Lamia Oy (https://lamia.fi)
 * @author    Szymon Nosal <simon@lamia.fi>
 */

use \Verifone\Core\DependencyInjection\Service\PaymentInfoImpl;
use \Verifone\Core\ServiceFactory;
use \Verifone\Core\ExecutorContainer;
use \Verifone\Core\DependencyInjection\Transporter\CoreResponse;
use \Verifone\Core\Executor\BackendServiceExecutor;
use Verifone\Core\Service\Backend\RemoveSavedCreditCardsService;


class Verifone_Core_PaymentMethods extends Verifone_Core
{

    /**
     * Refresh payment methods from Verifone
     *
     * @return array|null
     */
    public static function refreshAvailablePaymentMethods()
    {
        $config = Verifone_Config::getInstance();
        $configObject = self::getAvailablePaymentMethodsConfigObject();

        /** @var GetAvailablePaymentMethodsService $service */
        $service = ServiceFactory::createService($configObject, 'Backend\GetAvailablePaymentMethodsService');
        $container = new ExecutorContainer();

        /** @var BackendServiceExecutor $exec */
        $exec = $container->getExecutor('backend');

        /** @var CoreResponse $response */
        $response = $exec->executeService($service, $config->getPaymentPublicKeyFile());

        if (!$response->getStatusCode()) {
            return null;
        }

        $body = $response->getBody();
        $methods = ['all'];

        /** @var PaymentMethod $item */
        foreach ($body as $item) {
            $methods[] = $item->getCode();
        }

        self::_savePaymentMethods($methods);

        return $methods;
    }

    /**
     * Save fetched payment methods
     *
     * @param $methods
     */
    protected static function _savePaymentMethods($methods)
    {
        Verifone_PaymentMethods::updatePaymentMethods($methods);
    }

    /**
     * Delete card in Verifone System
     *
     * @param $cardId
     * @return array|null
     * @throws Exception
     */
    public static function deleteCard($cardId)
    {
        $config = Verifone_Config::getInstance();

        $customerData = Verifone_Data::collectCustomerData();
        if ($customerData === null) {
            return null;
        }

        $customer = self::createCustomerObject($customerData);

        $payment = new PaymentInfoImpl('', '', $cardId);

        try {
            /**
             * @var RemoveSavedCreditCardsService $service
             */
            $service = ServiceFactory::createService(self::getBackedConfigObject(), 'Backend\RemoveSavedCreditCardsService');
            $service->insertCustomer($customer);
            $service->insertPaymentInfo($payment);

            $container = new ExecutorContainer();

            /** @var BackendServiceExecutor $exec */
            $exec = $container->getExecutor('backend');
            $response = $exec->executeService($service, $config->getPaymentPublicKeyFile());

            return $response;
        } catch (\Exception $e) {
            throw $e;
        }
    }

}