<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is released under commercial license by Lamia Oy.
 *
 * @copyright Copyright (c) 2017 Lamia Oy (https://lamia.fi)
 * @author    Szymon Nosal <simon@lamia.fi>
 */

use \Verifone\Core\DependencyInjection\Service\TransactionImpl;
use \Verifone\Core\Service\Backend\RefundPaymentService;
use \Verifone\Core\ServiceFactory;
use \Verifone\Core\ExecutorContainer;
use \Verifone\Core\Executor\BackendServiceExecutor;
use \Verifone\Core\DependencyInjection\Transporter\CoreResponse;

class Verifone_Core_Refund extends Verifone_Core
{
    /**
     * Method for make refund request in Verifone
     *
     * @param $order
     * @param $amount
     * @return bool|CoreResponse
     * @throws Exception
     */
    public static function refund($order, $amount)
    {
        $config = Verifone_Config::getInstance();

        $configObject = self::getBackedConfigObject();

        $refundAmount = $amount * 100;

        $transaction = new TransactionImpl(
            $order['payment_method'],
            $order['order_ext_id'],
            (string)$refundAmount,
            Verifone_System::getCurrencyNumber()
        );

        try {
            /** @var RefundPaymentService $service */
            $service = ServiceFactory::createService($configObject, 'Backend\RefundPaymentService');
            $service->insertTransaction($transaction);
            $service->insertRefundProduct($transaction);

            $container = new ExecutorContainer();

            /** @var BackendServiceExecutor $exec */

            $exec = $container->getExecutor('backend');

            /** @var CoreResponse $response */
            $response = $exec->executeService($service, $config->getPaymentPublicKeyFile());

            if ($response->getStatusCode()) {
                return $response;
            }
        } catch (\Exception $e) {
            throw $e;
        }

        return false;
    }
}