<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is released under commercial license by Lamia Oy.
 *
 * @copyright Copyright (c) 2017 Lamia Oy (https://lamia.fi)
 * @author    Szymon Nosal <simon@lamia.fi>
 */

use \Verifone\Core\DependencyInjection\Service\OrderImpl;
use \Verifone\Core\DependencyInjection\Service\PaymentInfoImpl;
use \Verifone\Core\Service\Frontend\AddNewCardService;
use \Verifone\Core\ServiceFactory;
use \Verifone\Core\ExecutorContainer;
use \Verifone\Core\DependencyInjection\Transporter\CoreResponse;
use \Verifone\Core\Service\Backend\GetSavedCreditCardsService;
use \Verifone\Core\Executor\BackendServiceExecutor;
use \Verifone\Core\DependencyInjection\CoreResponse\CardImpl;
use Verifone\Core\Service\Backend\RemoveSavedCreditCardsService;

class Verifone_Core_SavedPaymentMethods extends Verifone_Core
{
    public static function prepareNewCardRequestFormData()
    {

        $config = Verifone_Config::getInstance();

        $urls = self::getRedirectUrlsObject(true);

        $customerData = Verifone_Data::collectCustomerData();
        $customer = self::createCustomerObject($customerData);

        $order = new OrderImpl(
            'addNewCard',
            gmdate('Y-m-d H:i:s'),
            Verifone_System::getCurrencyNumber(),
            '1',
            '1',
            '0'
        );

        $payment = new PaymentInfoImpl(
            $config->getPaymentPageLanguage(),
            PaymentInfoImpl::SAVE_METHOD_SAVE_ONLY,
            '',
            (string)time(),
            (bool)$config->isSaveMaskedPanNumber()
        );

        /** @var AddNewCardService $service */
        $service = ServiceFactory::createService(self::getFrontendConfigObject($urls), 'Frontend\AddNewCardService');
        $service->insertCustomer($customer);
        $service->insertOrder($order);
        $service->insertPaymentInfo($payment);

        // for json: new ExecutorContainer(array('requestConversion.class' => ExecutorContainer::REQUEST_CONVERTER_TYPE_JSON));
        $container = new ExecutorContainer();
        $exec = $container->getExecutor(ExecutorContainer::EXECUTOR_TYPE_FRONTEND);

        $form = $exec->executeService($service, Verifone_Urls::getUrls('page'), $config->isValidateUrl());

        return $form;

    }

    public static function verifyAddNewCardResponse($status)
    {
        $result = [
            'success' => $status == 'success',
            'message' => self::_getResponseMessage($status)
        ];

        return $result;

    }

    protected static function _getResponseMessage($status)
    {
        $messages = [
            'cancel' => 'Card adding canceled.',
            'error' => 'Server error, please try again later.',
            'expired' => 'Your card has expired.',
            'rejected' => 'Your card has been rejected.',
            'success' => 'Your card has been successfully added.'
        ];

        if (isset($messages[$status])) {
            return $messages[$status];
        }

        return $messages['error'];

    }

    /**
     * @return CoreResponse|null
     * @throws Exception
     */
    public static function getListSavedPaymentMethods()
    {
        $config = Verifone_Config::getInstance();

        $customerData = Verifone_Data::collectCustomerData();
        if ($customerData === null) {
            return null;
        }

        $customer = self::createCustomerObject($customerData);

        try {
            /**
             * @var GetSavedCreditCardsService $service
             */
            $service = ServiceFactory::createService(self::getBackedConfigObject(), 'Backend\GetSavedCreditCardsService');
            $service->insertCustomer($customer);

            $container = new ExecutorContainer();

            /** @var BackendServiceExecutor $exec */
            $exec = $container->getExecutor('backend');

            /** @var CoreResponse $response */
            $response = $exec->executeService($service, $config->getPaymentPublicKeyFile());

            return $response;
        } catch (\Exception $e) {
            return null;
        }
    }

    /**
     * Delete card in Verifone System
     *
     * @param $cardId
     * @return array|null
     * @throws Exception
     */
    public static function deleteCard($cardId)
    {
        $config = Verifone_Config::getInstance();

        $customerData = Verifone_Data::collectCustomerData();
        if ($customerData === null) {
            return null;
        }

        $customer = self::createCustomerObject($customerData);

        $payment = new PaymentInfoImpl('', '', $cardId);

        try {
            /**
             * @var RemoveSavedCreditCardsService $service
             */
            $service = ServiceFactory::createService(self::getBackedConfigObject(), 'Backend\RemoveSavedCreditCardsService');
            $service->insertCustomer($customer);
            $service->insertPaymentInfo($payment);

            $container = new ExecutorContainer();

            /** @var BackendServiceExecutor $exec */
            $exec = $container->getExecutor('backend');
            $response = $exec->executeService($service, $config->getPaymentPublicKeyFile());

            return $response;
        } catch (\Exception $e) {
            throw $e;
        }
    }


}