<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is released under commercial license by Lamia Oy.
 *
 * @copyright Copyright (c) 2018 Lamia Oy (https://lamia.fi)
 */


class Verifone_Summary
{

    protected static $_language = null;


    protected static function _getLanguage()
    {
        if(null === self::$_language) {
            $storage = Verifone_Storage::getInstance();
            self::$_language = $storage->getLanguageModel();
        }

        return self::$_language;
    }

    public static function getConfigurationDataForDisplay()
    {

        $config = Verifone_Config::getInstance();

        /** Data for display */
        $display = array();

        $display['isLiveMode'] = array(
            'label' => self::_getLanguage()->get('summary_mode'),
            'value' => $config->isLiveMode() ? self::_getLanguage()->get('entry_option_live') : self::_getLanguage()->get('entry_option_test'),
            'has_desc' => false,
            'has_desc_class' => false
        );

        $display['merchantCode'] = array(
            'label' => self::_getLanguage()->get('summary_merchant_aggrement_code'),
            'value' => $config->getMerchantAgreement(),
            'has_desc' => false,
            'has_desc_class' => false
        );
        if ($config->getMerchantAgreement() === $config->getMerchantAgreementDefault()) {
            $display['merchantCode']['desc'] = self::_getLanguage()->get('summary_merchant_aggrement_code');
            $display['merchantCode']['desc_class'] = 'info';
            $display['merchantCode']['has_desc'] = true;
            $display['merchantCode']['has_desc_class'] = true;
        }

        $display['delayedUrl'] = array(
            'label' => self::_getLanguage()->get('entry_title_delayed_url'),
            'value' => Verifone_Urls::getPaymentDelayedLink(),
            'desc' => self::_getLanguage()->get('summary_delayed_url_desc'),
            'desc_class' => 'success',
            'has_desc' => true,
            'has_desc_class' => true
        );

        $display['keyHandlingMode'] = array(
            'label' => self::_getLanguage()->get('summary_key_handling_mode'),
            'value' => $config->isKeySimpleMode() ? self::_getLanguage()->get('entry_option_key_simple') : self::_getLanguage()->get('entry_option_key_advanced'),
            'has_desc' => false,
            'has_desc_class' => false
        );

        $display['paymentServiceKey'] = array(
            'label' => self::_getLanguage()->get('summary_payment_service_key'),
            'value' => $config->getPaymentPublicKeyPath(),
            'has_desc' => false,
            'has_desc_class' => false
        );

        if (file_exists($config->getPaymentPublicKeyPath())) {
            $display['paymentServiceKey']['desc'] = self::_getLanguage()->get('summary_key_available');
            $display['paymentServiceKey']['desc_class'] = 'success';
            $display['paymentServiceKey']['has_desc'] = true;
            $display['paymentServiceKey']['has_desc_class'] = true;
        } else {
            $display['paymentServiceKey']['desc'] = self::_getLanguage()->get('summary_key_load_problem');
            $display['paymentServiceKey']['desc_class'] = 'success';
            $display['paymentServiceKey']['has_desc'] = true;
            $display['paymentServiceKey']['has_desc_class'] = true;
        }

        if ($config->isKeyAdvancedMode()) {

            $path = $config->getKeysDirectory();

            $display['directory'] = array(
                'label' => self::_getLanguage()->get('entry_title_keys_directory'),
                'value' => $path,
                'has_desc' => false,
                'has_desc_class' => false
            );
            if (file_exists($path) && is_writable($path)) {
                $display['directory']['desc'] = self::_getLanguage()->get('summary_directory_configured');
                $display['directory']['desc_class'] = 'success';
                $display['directory']['has_desc'] = true;
                $display['directory']['has_desc_class'] = true;
            } else {
                $display['directory']['desc'] = self::_getLanguage()->get('summary_directory_configured_error');
                $display['directory']['desc_class'] = 'error';
                $display['directory']['has_desc'] = true;
                $display['directory']['has_desc_class'] = true;
            }

            if($config->getMerchantAgreementDefault() !== $config->getMerchantAgreement()) {
                $display['shopPrivateKey'] = array(
                    'label' => self::_getLanguage()->get('summary_path_private_key'),
                    'value' => $config->getShopPrivateKeyPath(),
                    'has_desc' => false,
                    'has_desc_class' => false
                );

                if (file_exists($display['shopPrivateKey']['value']) && !empty($config->getShopPrivateKeyFileName())) {
                    $display['shopPrivateKey']['desc'] = self::_getLanguage()->get('summary_key_available');
                    $display['shopPrivateKey']['desc_class'] = 'success';
                    $display['shopPrivateKey']['has_desc'] = true;
                    $display['shopPrivateKey']['has_desc_class'] = true;
                } else {
                    $display['shopPrivateKey']['desc'] = self::_getLanguage()->get('summary_key_not_available');
                    $display['shopPrivateKey']['desc_class'] = 'error';
                    $display['shopPrivateKey']['has_desc'] = true;
                    $display['shopPrivateKey']['has_desc_class'] = true;
                }
            } else {
                $display['shopPrivateKey'] = array(
                    'label' => self::_getLanguage()->get('summary_path_private_key'),
                    'value' => '',
                    'has_desc' => true,
                    'has_desc_class' => true,
                    'desc' => self::_getLanguage()->get('summary_default_key'),
                    'desc_class' => 'info'
                );
            }
        } else {

            $display['shopPrivateKey'] = array(
                'label' => self::_getLanguage()->get('summary_path_private_key'),
                'value' => 'Key file stored in database',
                'has_desc' => false,
                'has_desc_class' => false
            );

            if ($config->getShopPrivateKey() !== null && $config->getShopPrivateKey() !== $config->getShopPrivateKeyDefault()) {
                $display['shopPrivateKey']['desc'] = self::_getLanguage()->get('summary_key_available');
                $display['shopPrivateKey']['desc_class'] = 'success';
                $display['shopPrivateKey']['has_desc'] = true;
                $display['shopPrivateKey']['has_desc_class'] = true;
            } elseif(!$config->isLiveMode() && $config->getMerchantAgreement() === $config->getMerchantAgreementDefault()) {
                $display['shopPrivateKey']['desc'] = self::_getLanguage()->get('summary_default_key');
                $display['shopPrivateKey']['desc_class'] = 'info';
                $display['shopPrivateKey']['has_desc'] = true;
                $display['shopPrivateKey']['has_desc_class'] = true;
            } else {
                $display['shopPrivateKey']['desc'] = self::_getLanguage()->get('summary_key_fetch_problem_private');
                $display['shopPrivateKey']['desc_class'] = 'error';
                $display['shopPrivateKey']['has_desc'] = true;
                $display['shopPrivateKey']['has_desc_class'] = true;
            }
        }

        if ($config->isKeySimpleMode() && $config->getShopPublicKey() === null && $config->getMerchantAgreement() !== $config->getMerchantAgreementDefault()) {
            $display['shopPublicKeyContent'] = array(
                'label' => self::_getLanguage()->get('summary_public_key'),
                'value' => '',
                'has_desc' => true,
                'has_desc_class' => true,
                'desc' => self::_getLanguage()->get('summary_key_fetch_problem_public'),
                'desc_class' => 'error'
            );
        } elseif ($config->getShopPublicKey() !== null) {
            $display['shopPublicKeyContent'] = array(
                'label' => self::_getLanguage()->get('summary_public_key'),
                'value' => $config->getShopPublicKey(),
                'has_desc' => true,
                'has_desc_class' => true,
                'desc' => self::_getLanguage()->get('summary_key_copy'),
                'desc_class' => 'success'
            );
        }

        return $display;
    }
}