<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is released under commercial license by Lamia Oy.
 *
 * @copyright Copyright (c) 2017 Lamia Oy (https://lamia.fi)
 * @author    Szymon Nosal <simon@lamia.fi>
 */

class Verifone_Config
{

    const ONE_CLICK_MAX_AMOUNT = 100;

    protected $_objectData = [];

    protected $_settingsObject = null;
    protected $_settingsData = null;

    /**
     * @var Verifone_Config $instance Singleton The reference the *Singleton* instance of this class
     */
    private static $instance;

    /**
     * Returns the *Singleton* instance of this class.
     *
     * @return Verifone_Config Singleton The *Singleton* instance.
     */
    public static function getInstance()
    {
        if (null === self::$instance) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    /**
     * Private clone method to prevent cloning of the instance of the
     * *Singleton* instance.
     *
     * @return void
     */
    private function __clone()
    {
    }

    /**
     * Private wakeup method to prevent unserializing of the *Singleton*
     * instance.
     *
     * @return void
     */
    private function __wakeup()
    {
    }

    /**
     * Protected constructor to prevent creating a new instance of the
     * *Singleton* via the `new` operator from outside of this class.
     */
    protected function __construct()
    {
        $this->_settingsObject = Verifone_Storage::getInstance()->getSettingModel();
        $this->_settingsData = $this->_settingsObject->getSetting('payment_verifone');
    }

    /**
     * @param $key
     * @return mixed|null
     */
    public function __get($key)
    {
        if (isset($this->_objectData[$key])) {
            return $this->_objectData[$key];
        } else {
            return null;
        }
    }

    /**
     * @param $key
     * @param $value
     * @return $this
     */
    public function __set($key, $value)
    {
        $this->_objectData[$key] = $value;
        return $this;
    }

    /**
     * Method for get data from settings. if exist get from current object, if not fetch from settings.
     *
     * @param $key
     * @return mixed|null
     */
    public function getData($key)
    {

        if($this->$key === null) {
            if(isset($this->_settingsData['payment_verifone_' . $key])) {
                $this->$key = $this->_settingsData['payment_verifone_' . $key];
            } elseif($this->getDefault($key) !== null){
                $this->$key = $this->getDefault($key);
            } else {
                return null;
            }
        }

        return $this->$key;
    }

    protected function _getTestLiveOption($key)
    {

        if ($this->isLiveMode()) {
            return $this->getData($key);
        } elseif (!empty($this->getData($key . '_test'))) {
            return $this->getData($key . '_test');
        } else {
            return $this->getDefault($key);
        }
    }

    public function getDefault($key)
    {
        $settingsObject = new Verifone_Settings();
        return $settingsObject->getDefaultValue($key);
    }

    /** FIELDS */

    public function isEnabled()
    {
        return (bool)$this->getData('status');
    }

    public function getMerchantAgreement()
    {

        if ($this->isLiveMode()) {
            return $this->getData('merchant_agreement_code');
        }

        if (!empty($this->getData('merchant_agreement_code_test'))) {
            return $this->getData('merchant_agreement_code_test');
        }

        return $this->getDefault('merchant_agreement_code_test');

    }

    public function getMerchantAgreementDefault()
    {
        return $this->getDefault('merchant_agreement_code_test');
    }

    public function isLiveMode()
    {
        return (bool)$this->getData('is_live_mode');
    }

    public function getPayPageUrl($index)
    {
        return $this->getData('pay_page_url_' . $index);
    }

    public function getPaymentPageLanguage()
    {
        return $this->getData('payment_page_language');
    }

    public function isValidateUrl()
    {
        return (bool)$this->getData('validate_url');
    }

    public function isSkipConfirmationPage()
    {
        return (bool)$this->getData('skip_confirmation_page');
    }

    public function getBasketItemSending()
    {
        return $this->getData('basket_item_sending');
    }

    public function isCombineInvoiceBasketItems()
    {
        return (bool)$this->getData('combine_invoice_basket_items');
    }

    public function getExternalCustomerIdField()
    {
        return $this->getData('external_customer_id_field');
    }

    /**
     * @return array
     */
    public function getPaymentMethods()
    {
        return $this->getData('payment_methods') ? : [];
    }

    public function isAllowToSaveCC()
    {
        return (bool)$this->getData('allow_to_save_cc');
    }

    public function isSaveMaskedPanNumber()
    {
        return (bool)$this->getData('save_masked_pan_number');
    }

    public function getRememberCCInfo()
    {
        return $this->getData('remember_cc_info');
    }

    public function getMinOrderTotal()
    {
        return $this->getData('min_order_total');
    }

    public function getMaxOrderTotal()
    {
        return $this->getData('max_order_total');
    }

    public function isDisableRsaBlinding()
    {
        return (bool)$this->getData('disable_rsa_blinding');
    }

    public function getCurrency()
    {
        return $this->_settingsObject->getSettingValue('config_currency');
    }

    public function isSecure()
    {
        return (bool)$this->_settingsObject->getSettingValue('config_secure');
    }

    public function getOrderStatusPendingPayment()
    {
        return $this->getData('order_status_id_pending_payment');
    }

    public function getOrderStatusSuccess()
    {
        return $this->getData('order_status_id_success');
    }

    public function getOrderStatusFail()
    {
        return $this->getData('order_status_id_fail');
    }

    public function getStyleCode()
    {
        return $this->getData('style_code');
    }

    public function getKeyMode()
    {
        return $this->getData('key_handling_mode');
    }

    public function isKeySimpleMode()
    {
        return (int)$this->getKeyMode() == 0;
    }

    public function isKeyAdvancedMode()
    {
        return (int)$this->getKeyMode() == 1;
    }

    /** FILES */
    public function getKeysDirectory()
    {
        return $this->getData('keys_directory');
    }

    public function getShopPrivateKeyFileName()
    {
        if($this->isLiveMode()) {
            return $this->getData('shop_private_keyfile');
        }

        return $this->getData('shop_private_keyfile_test');
    }

    public function getLiveShopPrivateKeyPath()
    {

        if ($this->isKeySimpleMode()) {
            return null;
        }

        return $this->getKeysDirectory() . DIRECTORY_SEPARATOR . $this->getData('shop_private_keyfile');

    }

    public function getTestShopPrivateKeyPath()
    {

        if ($this->isKeySimpleMode()) {
            return null;
        }

        return $this->getKeysDirectory() . DIRECTORY_SEPARATOR . $this->getData('shop_private_keyfile_test');
    }

    public function getShopPrivateKeyPath()
    {
        if($this->isLiveMode()) {
            return $this->getLiveShopPrivateKeyPath();
        }

        return $this->getTestShopPrivateKeyPath();
    }

    public function getShopPrivateKeyFile()
    {
        return $this->getShopPrivateKey();
    }

    public function getShopPrivateKey()
    {

        // If TEST mode is set
        if (!$this->isLiveMode()) {

            if ($this->getMerchantAgreement() === $this->getMerchantAgreementDefault()) {
                // If DEFAULT test merchant is set, return default key
                return $this->getShopPrivateKeyDefault();
            }

            if ($this->isKeySimpleMode()) {
                // If CUSTOM test merchant is set, and SIMPLE mode is set, return generated key stored in DB
                return Verifone_Keys::getTestPrivateKey();
            }

            $path = $this->getTestShopPrivateKeyPath();
            if (file_exists($path)) {
                // If CUSTOM test merchant is set, and ADVANCED mode is set, return old key stored in files
                return file_get_contents($path);
            }

            // return default key file
            return $this->getShopPrivateKeyDefault();
        }

        // If LIVE mode is set

        if ($this->isKeySimpleMode()) {
            // If CUSTOM test merchant is set, and SIMPLE mode is set, return generated key stored in DB
            return Verifone_Keys::getLivePrivateKey();
        }

        $path = $this->getLiveShopPrivateKeyPath();
        if (file_exists($path)) {
            // If CUSTOM test merchant is set, and ADVANCED mode is set, return old key stored in files
            return file_get_contents($path);
        }

        // return nothing
        return null;
    }

    public function getShopPrivateKeyPathDefault()
    {
        return $this->getModuleKeysDirectory() . Verifone_Keys::STORAGE_KEY_DEFAULT_PRIVATE;
    }

    public function getShopPrivateKeyDefault()
    {
        return file_get_contents($this->getShopPrivateKeyPathDefault());
    }

    public function getShopPublicKey()
    {
        // If TEST mode is set
        if (!$this->isLiveMode()) {

            if ($this->getMerchantAgreement() === $this->getMerchantAgreementDefault()) {
                // If DEFAULT test merchant is set, return default key
                // For default simple key is not require to configure in payment service.
                return null;
            }

            if ($this->isKeySimpleMode()) {
                // If CUSTOM test merchant is set, and SIMPLE mode is set, return generated key stored in DB
                return Verifone_Keys::getTestPublicKey();
            }


            // If CUSTOM test merchant is set, and ADVANCED mode is set, return old key stored in files
            // This is not required, because if this mode is set, it means that payment service is configured,
            // and it does not require to configure again.
            return null;
        }

        // If LIVE mode is set
        if ($this->isKeySimpleMode()) {
            // If CUSTOM test merchant is set, and SIMPLE mode is set, return generated key stored in DB
            return Verifone_Keys::getLivePublicKey();
        }

        // When advanced mode is set, or is default merchant agreement then in not require simple key to display.
        return null;
    }

    public function getShopPublicKeyPathDefault()
    {
        return $this->getModuleKeysDirectory() . Verifone_Keys::STORAGE_KEY_DEFAULT_PUBLIC;
    }

    public function getShopPublicKeyDefault()
    {
        return file_get_contents($this->getShopPublicKeyPathDefault());
    }

    public function getPaymentPublicKeyPath()
    {
        if($this->isLiveMode()) {
            return $this->getModuleKeysDirectory() . Verifone_Keys::STORAGE_KEY_GATEWAY_LIVE;
        }

        return $this->getModuleKeysDirectory() . Verifone_Keys::STORAGE_KEY_GATEWAY_TEST;
    }

    public function getPaymentPublicKeyFile()
    {
        return file_get_contents($this->getPaymentPublicKeyPath());
    }

    public function getModuleKeysDirectory()
    {
        return  DIR_SYSTEM . 'library' . DIRECTORY_SEPARATOR . 'verifone' . DIRECTORY_SEPARATOR . 'keys' . DIRECTORY_SEPARATOR;
    }


}