<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is released under commercial license by Lamia Oy.
 *
 * @copyright Copyright (c) 2017 Lamia Oy (https://lamia.fi)
 * @author    Szymon Nosal <simon@lamia.fi>
 */

use \Verifone\Core\DependencyInjection\Configuration\Backend\BackendConfigurationImpl;
use \Verifone\Core\DependencyInjection\Configuration\Backend\GetAvailablePaymentMethodsConfigurationImpl;
use \Verifone\Core\DependencyInjection\Configuration\Frontend\RedirectUrlsImpl;
use \Verifone\Core\DependencyInjection\Configuration\Frontend\FrontendConfigurationImpl;
use \Verifone\Core\DependencyInjection\Service\CustomerImpl;
use \Verifone\Core\DependencyInjection\Service\OrderImpl;
use \Verifone\Core\DependencyInjection\Service\AddressImpl;

class Verifone_Core
{
    /**
     * @return BackendConfigurationImpl
     */
    public static function getBackedConfigObject()
    {

        $config = Verifone_Config::getInstance();

        $configObject = new BackendConfigurationImpl(
            $config->getShopPrivateKeyFile(),
            $config->getMerchantAgreement(),
            Verifone_System::getSystemName(),
            Verifone_System::getVersion(),
            Verifone_Urls::getUrls('server'),
            $config->isDisableRsaBlinding()
        );

        return $configObject;
    }

    /**
     * @return GetAvailablePaymentMethodsConfigurationImpl
     */
    public static function getAvailablePaymentMethodsConfigObject()
    {
        $config = Verifone_Config::getInstance();

        $configObject = new GetAvailablePaymentMethodsConfigurationImpl(
            $config->getShopPrivateKeyFile(),
            $config->getMerchantAgreement(),
            Verifone_System::getSystemName(),
            Verifone_System::getVersion(),
            Verifone_Urls::getUrls('server'),
            Verifone_System::getCurrencyNumber(),
            $config->isDisableRsaBlinding()
        );

        return $configObject;
    }

    /**
     * @param RedirectUrlsImpl $urls
     * @return FrontendConfigurationImpl
     */
    public static function getFrontendConfigObject(RedirectUrlsImpl $urls)
    {
        $config = Verifone_Config::getInstance();

        $configObject = new FrontendConfigurationImpl(
            $urls,
            $config->getShopPrivateKeyFile(),
            $config->getMerchantAgreement(),
            Verifone_System::getSystemName(),
            Verifone_System::getVersion(),
            (string)$config->isSkipConfirmationPage(),
            $config->isDisableRsaBlinding(),
            $config->getStyleCode()
        );

        return $configObject;
    }

    /**
     * @param $customerData
     * @return CustomerImpl
     */
    public static function createCustomerObject($customerData, $address = null)
    {
        $customer = new CustomerImpl(
            self::sanitize($customerData['firstname']),
            self::sanitize($customerData['lastname']),
            self::sanitize($customerData['phone']),
            self::sanitize($customerData['email']),
            $address,
            isset($customerData['external_id']) && $customerData['external_id'] ? (string)$customerData['external_id'] : ''
        );

        return $customer;
    }

    public static function createAddressObject($addressData)
    {
        $address = new AddressImpl(
            $addressData['address_1'],
            $addressData['address_2'],
            '',
            $addressData['city'],
            $addressData['postcode'],
            $addressData['country_code'],
            $addressData['firstname'],
            $addressData['lastname'],
            $addressData['telephone'],
            $addressData['email']
        );

        return $address;
    }

    public static function createOrderObject($orderData)
    {
        $order = new OrderImpl(
            (string)$orderData['order_id'],
            $orderData['time'],
            (string)$orderData['currency_code'],
            (string)$orderData['total_incl_amount'],
            (string)$orderData['total_excl_amount'],
            (string)$orderData['total_tax']
        );

        return $order;
    }

    public static function getRedirectUrlsObject($isCard = false)
    {

        if (!$isCard) {
            $urls = new RedirectUrlsImpl(
                Verifone_Urls::getPaymentLink('success'),
                Verifone_Urls::getPaymentLink('rejected'),
                Verifone_Urls::getPaymentLink('cancel'),
                Verifone_Urls::getPaymentLink('expired'),
                Verifone_Urls::getPaymentLink('error')
            );
        } else {
            $link = Verifone_Urls::getCardLink();
            $urls = new RedirectUrlsImpl(
                self::addParameterToUrl('status', 'success', $link),
                self::addParameterToUrl('status', 'rejected', $link),
                self::addParameterToUrl('status', 'cancel', $link),
                self::addParameterToUrl('status', 'expired', $link),
                self::addParameterToUrl('status', '', $link)
            );
        }

        return $urls;
    }

    /**
     * Remove unexpected characters
     *
     * @param $string
     * @return mixed
     */
    public static function sanitize($string)
    {
        return str_replace('"', '', str_replace('\\', '', str_replace('-', ' ', $string)));
    }

    public static function addParameterToUrl($key, $value, $url)
    {
        if (strpos($url, '?')) {
            $url .= '&' . $key . '=' . $value;
        } else {
            $url .= '?' . $key . '=' . $value;
        }

        return $url;
    }
}