<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is released under commercial license by Lamia Oy.
 *
 * @copyright Copyright (c) 2017 Lamia Oy (https://lamia.fi)
 * @author    Szymon Nosal <simon@lamia.fi>
 */

class Verifone_PaymentMethods
{

    const TYPE_ALL = 'all';
    const TYPE_CARD = 'card';
    const TYPE_BANK = 'bank';
    const TYPE_INVOICE = 'invoice';
    const TYPE_ELECTRONIC = 'electronic';

    const PAYMENT_METHODS_CODE = 'payment_verifone_methods';
    const PAYMENT_METHODS_CODE_KEY = 'payment_verifone_methods_all';

    /**
     * Get array with all possible payment methods
     *
     * @return array
     */
    public static function getPaymentMethodsArray()
    {
        return [
            'all' => ['type' => self::TYPE_ALL, 'name' => 'ALL_IN_ONE'],
            'visa' => ['type' => self::TYPE_CARD, 'name' => 'VISA'],
            'master-card' => ['type' => self::TYPE_CARD, 'name' => 'MASTER_CARD'],
            'dankort' => ['type' => self::TYPE_CARD, 'name' => 'DANKORT'],
            'amex' => ['type' => self::TYPE_CARD, 'name' => 'AMEX'],
            'diners' => ['type' => self::TYPE_CARD, 'name' => 'DINERS'],
            's-pankki-verkkomaksu' => ['type' => self::TYPE_BANK, 'name' => 'S_PANKKI_VERKKOMAKSU'],
            'aktia-maksu' => ['type' => self::TYPE_BANK, 'name' => 'AKTIA_MAKSU'],
            'op-pohjola-verkkomaksu' => ['type' => self::TYPE_BANK, 'name' => 'OP_POHJOLA_VERKKOMAKSU'],
            'nordea-e-payment' => ['type' => self::TYPE_BANK, 'name' => 'NORDEA_E_PAYMENT'],
            'sampo-web-payment' => ['type' => self::TYPE_BANK, 'name' => 'SAMPO_WEB_PAYMENT'],
            'handelsbanken-e-payment' => ['type' => self::TYPE_BANK, 'name' => 'HANDELSBANKEN_E_PAYMENT'],
            'alandsbanken-e-payment' => ['type' => self::TYPE_BANK, 'name' => 'ALANDSBANKEN_E_PAYMENT'],
            'nordea-se-db' => ['type' => self::TYPE_BANK, 'name' => 'NORDEA_SE_DB'],
            'handelsbanken-se-db' => ['type' => self::TYPE_BANK, 'name' => 'HANDELSBANKEN_SE_DB'],
            'swedbank-se-db' => ['type' => self::TYPE_BANK, 'name' => 'SWEDBANK_SE_DB'],
            'seb-se-db' => ['type' => self::TYPE_BANK, 'name' => 'SEB_SE_DB'],
            'bank-axess' => ['type' => self::TYPE_BANK, 'name' => 'BANK_AXESS'],
            'nordea-dk-db' => ['type' => self::TYPE_BANK, 'name' => 'NORDEA_DK_DB'],
            'danske-netbetaling' => ['type' => self::TYPE_BANK, 'name' => 'DANSKE_NETBETALING'],
            'saastopankin-verkkomaksu' => ['type' => self::TYPE_BANK, 'name' => 'SAASTOPANKIN_VERKKOMAKSU'],
            'pop-pankin-verkkomaksu' => ['type' => self::TYPE_BANK, 'name' => 'POP_PANKIN_VERKKOMAKSU'],
            'oma-saastopankin-verkkomaksu' => ['type' => self::TYPE_BANK, 'name' => 'OMA_SAASTOPANKIN_VERKKOMAKSU'],
            'svea-webpay-installment' => ['type' => self::TYPE_INVOICE, 'name' => 'SVEA_WEBPAY_INSTALLMENT'],
            'svea-webpay-invoice' => ['type' => self::TYPE_INVOICE, 'name' => 'SVEA_WEBPAY_INVOICE'],
            'handelsbanken-se-account' => ['type' => self::TYPE_INVOICE, 'name' => 'HANDELSBANKEN_SE_ACCOUNT'],
            'handelsbanken-se-invoice' => ['type' => self::TYPE_INVOICE, 'name' => 'HANDELSBANKEN_SE_INVOICE'],
            'invoice-collector' => ['type' => self::TYPE_INVOICE, 'name' => 'INVOICE_COLLECTOR'],
            'euroloan-invoice' => ['type' => self::TYPE_INVOICE, 'name' => 'EUROLOAN_INVOICE'],
            'enterpay-invoice' => ['type' => self::TYPE_INVOICE, 'name' => 'ENTERPAY_INVOICE'],
            'paypal' => ['type' => self::TYPE_INVOICE, 'name' => 'PAYPAL'],
            'swish' => ['type' => self::TYPE_ELECTRONIC, 'name' => 'SWISH'],
            'siirto' => ['type' => self::TYPE_ELECTRONIC, 'name' => 'SIIRTO'],
            'afterpay-invoice' => ['type' => self::TYPE_INVOICE, 'name' => 'AFTERPAY_INVOICE'],
            'mobilepay' => ['type' => self::TYPE_BANK, 'name' => 'MOBILEPAY'],
            'vipps' => ['type' => self::TYPE_BANK, 'name' => 'VIPPS'],
            'masterpass' => ['type' => self::TYPE_BANK, 'name' => 'MASTERPASS'],
        ];
    }

    /**
     * Get payment method display name (translated)
     *
     * @param $code
     * @return string
     */
    public static function getPaymentMethodDisplayName($code)
    {
        $methods = self::getPaymentMethodsArray();

        $language = Verifone_Storage::getInstance()->getLanguageModel();

        if (isset($methods[$code])) {
            return $language->get($methods[$code]['name']);
        }

        return '';
    }

    /**
     * @param $code
     * @return string
     */
    public static function getPaymentMethodName($code)
    {
        $methods = self::getPaymentMethodsArray();

        if (isset($methods[$code])) {
            return $methods[$code]['name'];
        }

        return '';
    }

    /**
     * Get list with all payment methods configured in Verifone
     *
     * @return array
     */
    public static function getPaymentMethods()
    {
        $settingsObject = Verifone_Storage::getInstance()->getSettingModel();
        $methods = $settingsObject->getSetting(self::PAYMENT_METHODS_CODE);

        if (empty($methods) || empty($methods[self::PAYMENT_METHODS_CODE_KEY])) {
            return [];
        }

        return $methods[self::PAYMENT_METHODS_CODE_KEY];
    }

    public static function updatePaymentMethods($methods)
    {
        $settingsObject = Verifone_Storage::getInstance()->getSettingModel();

        $setting[self::PAYMENT_METHODS_CODE_KEY] = $methods;
        $settingsObject->editSetting(self::PAYMENT_METHODS_CODE, $setting);

    }

    public static function getSelectPaymentMethods()
    {
        $config = Verifone_Config::getInstance();
        return $config->getPaymentMethods();
    }

    /**
     * Get all available payment methods
     *
     * @return array
     */
    public static function getAvailablePaymentMethods()
    {
        $methods = [];
        $available = self::getPaymentMethodsArray();

        foreach (self::getSelectPaymentMethods() as $code) {

            if (!array_key_exists($code, $available)) {
                continue;
            }

            $methods[$code] = $available[$code];
            $methods[$code]['code'] = $code;
            $methods[$code]['displayName'] = self::getPaymentMethodDisplayName($code);
        }

        return $methods;
    }

    /**
     * Get payment method information
     *
     * @param $code
     * @return mixed|null
     */
    public static function getPaymentMethodByCode($code)
    {
        $methods = self::getAvailablePaymentMethods();

        if (isset($methods[$code])) {
            return $methods[$code];
        }

        return null;
    }

}