<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is released under commercial license by Lamia Oy.
 *
 * @copyright Copyright (c) 2017 Lamia Oy (https://lamia.fi)
 * @author    Szymon Nosal <simon@lamia.fi>
 */
class Verifone_Settings
{
    const DESC_TIP = false;
    public function getSettings()
    {
        $settings = array(
            'display_summary' => [
                'type' => 'verifone-display-summary',
                'description' => null,
                'desc_tip' => self::DESC_TIP,
            ],
            'status' => array(
                'type' => 'select',
                'default' => 0,
                'options' => $this->_yesNoOptions()
            ),
            'title' => array(
                'type' => 'text',
                'default' => 'Verifone Payment',
            ),
            'sort_order' => array(
                'type' => 'text',
                'default' => '1',
            ),
            'is_live_mode' => array(
                'type' => 'select',
                'tooltip' => true,
                'default' => 0,
                'options' => [
                    ['value' => 0, 'label' => 'entry_option_test'],
                    ['value' => 1, 'label' => 'entry_option_live']
                ]
            ),
            'merchant_agreement_code' => array(
                'type' => 'text',
                'tooltip' => true,
                'default' => 'demo-merchant-agreement'
            ),
            'merchant_agreement_code_test' => array(
                'type' => 'text',
                'tooltip' => true,
                'default' => 'demo-merchant-agreement'
            ),
            'key_handling_mode' => array(
                'type' => 'select',
                'default' => 0,
                'options' => [
                    ['value' => 0, 'label' => 'entry_option_key_simple'],
                    ['value' => 1, 'label' => 'entry_option_key_advanced']

                ]
            ),
            'generate_keys' => array(
                'type' => 'generate_keys',
                'class' => 'depends-key_handling_mode-0',
                'route' => 'extension/payment/verifone/ajaxGenerateKeys'
            ),
            'keys_directory' => array(
                'type' => 'text',
                'tooltip' => true,
                'default' => '',
                'class' => 'depends-key_handling_mode-1'
            ),
            'shop_private_keyfile' => array(
                'type' => 'text',
                'tooltip' => true,
                'default' => '',
                'class' => 'depends-key_handling_mode-1'
            ),
            'shop_private_keyfile_test' => array(
                'type' => 'text',
                'tooltip' => true,
                'default' => 'demo-merchant-agreement-private.pem',
                'class' => 'depends-key_handling_mode-1'
            ),
            'pay_page_url_1' => array(
                'type' => 'text',
                'tooltip' => true,
                'default' => 'https://epayment1.point.fi/pw/payment'
            ),
            'pay_page_url_2' => array(
                'type' => 'text',
                'tooltip' => true,
                'default' => 'https://epayment2.point.fi/pw/payment'
            ),
            'pay_page_url_3' => array(
                'type' => 'text',
                'tooltip' => true,
                'default' => ''
            ),
            'payment_page_language' => array(
                'type' => 'select',
                'tooltip' => true,
                'default' => 'fi_FI',
                'options' => Verifone_System::getLocaleOptions()
            ),
            'validate_url' => array(
                'type' => 'select',
                'tooltip' => true,
                'default' => 1,
                'options' => $this->_yesNoOptions()
            ),
            'skip_confirmation_page' => array(
                'type' => 'select',
                'tooltip' => true,
                'default' => 1,
                'options' => $this->_yesNoOptions()
            ),
            'style_code' => array(
                'type' => 'text',
                'tooltip' => true,
                'default' => ''
            ),
            'basket_item_sending' => array(
                'type' => 'select',
                'tooltip' => true,
                'default' => 1,
                'options' => Verifone_System::getBasketItemsOptions()
            ),
            'combine_invoice_basket_items' => array(
                'type' => 'select',
                'tooltip' => true,
                'default' => 0,
                'options' => $this->_yesNoOptions()
            ),
            'external_customer_id_field' => array(
                'type' => 'select',
                'tooltip' => true,
                'default' => '',
                'options' => Verifone_System::getCustomerCustomFields()
            ),
            'refresh_payments' => array(
                'type' => 'url',
                'class' => '#',
                'route' => 'extension/payment/verifone/ajaxRefreshMethods'
            ),
            'payment_methods' => array(
                'type' => 'multiselect',
                'options' => Verifone_System::getPaymentMethodsOptions()
            ),
            'allow_to_save_cc' => array(
                'type' => 'select',
                'default' => 0,
                'options' => $this->_yesNoOptions()
            ),
            'save_masked_pan_number' => array(
                'type' => 'select',
                'default' => 0,
                'options' => $this->_yesNoOptions()
            ),
            'remember_cc_info' => array(
                'type' => 'textarea',
                'tooltip' => true
            ),
            'min_order_total' => array(
                'type' => 'text',
            ),
            'max_order_total' => array(
                'type' => 'text',
            ),
            'disable_rsa_blinding' => array(
                'type' => 'select',
                'tooltip' => true,
                'default' => 0,
                'options' => $this->_yesNoOptions()
            ),
            'order_status_id_pending_payment' => array(
                'type' => 'select',
                'default' => 1,
                'options' => Verifone_System::getOrderStatusesOption()
            ),
            'order_status_id_success' => array(
                'type' => 'select',
                'default' => 2,
                'options' => Verifone_System::getOrderStatusesOption()
            ),
            'order_status_id_fail' => array(
                'type' => 'select',
                'default' => 7,
                'options' => Verifone_System::getOrderStatusesOption()
            ),
            'secret_token' => array(
                'type' => 'text',
                'tooltip' => true
            ),
            'cron_url' => array(
                'type' => 'text',
                'readonly' => true,
                'tooltip' => true
            )
        );
        return $settings;
    }
    public function getDefaultValue($key)
    {
        $settings = $this->getSettings();
        if (isset($settings[$key]) && isset($settings[$key]['default'])) {
            return $settings[$key]['default'];
        }
        return null;
    }
    protected function _yesNoOptions()
    {
        return [
            ['value' => 0, 'label' => 'entry_option_no'],
            ['value' => 1, 'label' => 'entry_option_yes'],
        ];
    }
}