<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is released under commercial license by Lamia Oy.
 *
 * @copyright Copyright (c) 2017 Lamia Oy (https://lamia.fi)
 * @author    Szymon Nosal <simon@lamia.fi>
 */

class Verifone_Urls
{

    const PAY_PAGE_URL_DEMO = 'https://epayment.test.point.fi/pw/payment';
    const SERVER_URL_DEMO = 'https://epayment.test.point.fi/pw/serverinterface';

    const SERVER_URL_1 = 'https://epayment1.point.fi/pw/serverinterface';
    const SERVER_URL_2 = 'https://epayment2.point.fi/pw/serverinterface';
    const SERVER_URL_3 = 'https://epayment3.point.fi/pw/serverinterface';

    const WC_API = 'wc-api';
    const GATEWAY_NAME = 'WC_Gateway_Verifone';
    const HTTPS = 'https';
    const HTTP = 'http';

    const ACTION_DELAYED_SUCCESS = 'delayedSuccess';
    const ACTION_ADD_NEW_CARD = 'verifoneCardResponse';

    /**
     * Get urls to payment and server service for test and live environments.
     *
     * @param $type [server, page]
     * @return array
     */
    public static function getUrls($type)
    {
        $config = Verifone_Config::getInstance();
        $isLive = $config->isLiveMode();

        if ($isLive) {

            if ($type == 'server') {
                return [self::SERVER_URL_1, self::SERVER_URL_2, self::SERVER_URL_3];
            } elseif ($type == 'page') {
                $urls = [];
                for ($i = 1; $i <= 3; $i++) {
                    $url = $config->getPayPageUrl($i);
                    if (isset($url) && !empty($url)) {
                        $urls[] = $url;
                    }
                }
                return $urls;
            }

        } else {

            if ($type == 'server') {
                return [self::SERVER_URL_DEMO];
            } elseif ($type == 'page') {
                return [self::PAY_PAGE_URL_DEMO];
            }

        }

        return [];
    }

    /**
     * Check if shop is with ssl
     *
     * @return bool
     */
    protected static function _isSsl()
    {
        return (isset($_SERVER['HTTP_X_FORWARDED_PROTO']) && $_SERVER['HTTP_X_FORWARDED_PROTO'] === 'https') || (Verifone_Config::getInstance()->isSecure());
    }

    /**
     *  Get url for redirect to send payment request form page.
     *  Get url for set to payment request for fetch response.
     *
     * @return mixed
     */
    public static function getPaymentLink($status)
    {
        $storage = Verifone_Storage::getInstance();
        $query = $storage->getUrlModel()->link('extension/payment/verifone/' . $status, '', true);

        return $query;

    }

    /**
     * Get url for set to card request for fetch response.
     *
     * @return string
     */
    public static function getCardLink()
    {
        return '';
    }

    /**
     * Get url for set in Verifone Payment system for fetch delayed success response.
     *
     * @return string
     */
    public static function getPaymentDelayedLink()
    {
        return HTTPS_CATALOG . 'index.php?route=extension/payment/verifone/successDelayed';
    }

    public static function getCheckoutUrl()
    {
        $storage = Verifone_Storage::getInstance();
        $query = $storage->getUrlModel()->link('checkout/checkout', '', true);

        return $query;
    }

}